import os
import unittest

import pomsets.builder as BuilderModule
import pomsets.context as ContextModule
import pomsets.definition as DefinitionModule



def createTestPomset1(builder):

    compositeContext = builder.createNewNestPomset(name='composite')
    compositeDefinition = compositeContext.pomset()

    atomicContext = builder.createNewAtomicPomset(name='atomic')
    atomicDefinition = atomicContext.pomset()

    builder.addPomsetParameter(
        atomicDefinition, 'input file', {'direction':'input'})
    builder.addPomsetParameter(
        atomicDefinition, 'output file', {'direction':'input'})

    # setup the reference definition for parameter sweep
    mapperNode = compositeDefinition.createNode(id='node')
    mapperNode.definitionToReference(atomicDefinition)
    mapperNode.isCritical(True)
    mapperNode.name('mapper')

    compositeDefinition.name('basic map-reduce')
    
    return compositeDefinition



class TestDefinition1(unittest.TestCase):

    def setUp(self):

        self.builder = BuilderModule.Builder()

        pomset = createTestPomset1(self.builder)
        self.pomset = pomset

        return


    def tearDown(self):
        return


    def testGetParameter1(self):
        pomset = self.pomset
        self.assertRaises(NotImplementedError,
                          pomset.getParameter, 'foo')
        return


    def testGetParameter2(self):
        pomset = self.pomset

        self.builder.addPomsetParameter(
            pomset, 'foo', 
            {'direction':'input'})
        self.builder.addPomsetParameter(
            pomset, 'foo', 
            {'direction':'input'})

        self.assertRaises(NotImplementedError,
                          pomset.getParameter, 'foo')

        return


    def testGetParameter3(self):
        pomset = self.pomset

        self.builder.addPomsetParameter(
            pomset, 'foo', 
            {'direction':'input'})


        return


    def testParameterIsInOwnParameterSweepGroup(self):
        
        pomset = self.pomset

        nodes = [x for x in pomset.nodes() if x.id() == 'node']
        node = nodes[0]

        for parameterId in ['input file', 'output file']:
            self.assertTrue(
                node.parameterIsInOwnParameterSweepGroup(parameterId))
            node.isParameterSweep(parameterId, value=True)

        node.addParameterSweepGroup(['input file', 'output file'])
        for parameterId in ['input file', 'output file']:
            self.assertFalse(
                node.parameterIsInOwnParameterSweepGroup(parameterId))

        return


    def testGetParameterSweepGroup(self):

        pomset = self.pomset

        nodes = [x for x in pomset.nodes() if x.id() == 'node']
        node = nodes[0]

        for parameterId in ['input file', 'output file']:
            self.assertTrue(
                node.parameterIsInOwnParameterSweepGroup(parameterId))
            #self.assertEquals(node.getGroupForParameterSweep(parameterId),
            #                  tuple([parameterId]))

        for parameterId in ['input file', 'output file']:
            node.isParameterSweep(parameterId, value=True)

        node.addParameterSweepGroup(['input file', 'output file'])

        self.assertEquals(node.getGroupForParameterSweep('input file'),
                          node.getGroupForParameterSweep('output file'))

        return

    def testRemoveFromParameterSweepGroup(self):

        pomset = self.pomset

        nodes = [x for x in pomset.nodes() if x.id() == 'node']
        node = nodes[0]

        for parameterId in ['input file', 'output file']:
            node.isParameterSweep(parameterId, value=True)

        node.addParameterSweepGroup(['input file', 'output file'])

        for parameterId in ['input file', 'output file']:
            self.assertFalse(
                node.parameterIsInOwnParameterSweepGroup(parameterId))

        group = node.getGroupForParameterSweep('input file')
        node.removeParameterSweepGroup(group)

        for parameterId in ['input file', 'output file']:
            self.assertTrue(
                node.parameterIsInOwnParameterSweepGroup(parameterId))

        self.assertEquals(0,
                          node.parameterSweepGroups().rowCount())

        return

    def testAddToParameterSweepGroup(self):

        pomset = self.pomset

        nodes = [x for x in pomset.nodes() if x.id() == 'node']
        node = nodes[0]

        self.assertRaises(
            NotImplementedError,
            node.addParameterSweepGroup,
            ['input file', 'output file'])

        for parameterId in ['input file', 'output file']:
            self.assertTrue(
                node.parameterIsInOwnParameterSweepGroup(parameterId))

        for parameterId in ['input file', 'output file']:
            node.isParameterSweep(parameterId, value=True)

        node.addParameterSweepGroup(['input file', 'output file'])

        self.assertEquals(2,
                          node.parameterSweepGroups().rowCount())
        
        for parameterId in ['input file', 'output file']:
            self.assertFalse(
                node.parameterIsInOwnParameterSweepGroup(parameterId))
        self.assertEquals(node.getGroupForParameterSweep('input file'),
                          node.getGroupForParameterSweep('output file'))


        return


    # END class TestDefinition1
    pass


class TestDefinition2(unittest.TestCase):

    TEST_DATA_PATH = os.path.sep.join(['resources', 'testdata', 'TestDefinition', 'render.pomset'])

    def setUp(self):

        # load pomset from path
        pomsetContext = ContextModule.loadPomset(
            path=TestDefinition2.TEST_DATA_PATH)

        self.pomset = pomsetContext.pomset()
        return


    def assertPredecessors(self, node, expectedPredecessorNames):

        predecessors = [x for x in node.predecessors()]

        self.assertEquals(len(expectedPredecessorNames),
                          len(predecessors))

        actualPredecessorNames = [x.name() for x in predecessors]

        for expectedName in expectedPredecessorNames:
            self.assertTrue(expectedName in actualPredecessorNames)
        return


    def assertSuccessors(self, node, expectedSuccessorNames):
        successors = [x for x in node.successors()]

        self.assertEquals(len(expectedSuccessorNames),
                          len(successors))

        actualSuccessorNames = [x.name() for x in successors]

        for expectedName in expectedSuccessorNames:
            self.assertTrue(expectedName in actualSuccessorNames)
        return


    def testOrdering(self):

        nodes = [x for x in self.pomset.nodes() 
                 if x.name() == 'generate tiling info']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]
        self.assertPredecessors(node, [])
        self.assertSuccessors(node, ['generate tile list'])

        nodes = [x for x in self.pomset.nodes() 
                 if x.name() == 'generate tile list']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]
        self.assertPredecessors(node, ['generate tiling info'])
        self.assertSuccessors(node, ['read tile image list',
                                     'read tile index list'])

        nodes = [x for x in self.pomset.nodes() 
                 if x.name() == 'read tile index list']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]
        self.assertPredecessors(node, ['generate tile list'])
        self.assertSuccessors(node, ['render tile'])

        nodes = [x for x in self.pomset.nodes() 
                 if x.name() == 'read tile image list']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]
        self.assertPredecessors(node, ['generate tile list'])
        self.assertSuccessors(node, ['render tile'])

        nodes = [x for x in self.pomset.nodes() 
                 if x.name() == 'render tile']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]
        self.assertPredecessors(node, ['read tile index list',
                                       'read tile image list'])
        self.assertSuccessors(node, ['comp tiles'])


        nodes = [x for x in self.pomset.nodes() 
                 if x.name() == 'comp tiles']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]
        self.assertPredecessors(node, ['render tile'])
        self.assertSuccessors(node, [])

        return

    def testGetMinimalNodes(self):

        minimalNodes = self.pomset.getMinimalNodes()
        self.assertTrue(len(minimalNodes) is 1)

        self.assertEquals(['generate tiling info'],
                          [x.name() for x in minimalNodes])

        return


    def testGetMaximalNodes(self):
        
        maximalNodes = self.pomset.getMaximalNodes()
        self.assertTrue(len(maximalNodes) is 1)
        
        self.assertEquals(['comp tiles'],
                          [x.name() for x in maximalNodes])

        return


    # END class TestDefinition2
    pass


class TestParameterToEdit1(unittest.TestCase):

    TEST_DATA_PATH = os.path.sep.join(['resources', 'testdata', 'TestDefinition', 'render.pomset'])

    def setUp(self):

        # load pomset from path
        pomsetContext = ContextModule.loadPomset(
            path=TestParameterToEdit1.TEST_DATA_PATH)

        self.pomset = pomsetContext.pomset()
        return


    def testParameterToEdit1(self):

        nodes = [x for x in self.pomset.nodes() 
                 if x.name() == 'read tile index list']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]

        parameterId = 'eval result'
        expectedParameterId = 'eval result'

        # for an output parameter, itself
        nodeToEdit, parameterToEdit = node.getParameterToEdit(parameterId)

        self.assertTrue(nodeToEdit is node)
        self.assertEquals(expectedParameterId, parameterToEdit.id())

        return


    def testParameterToEdit2(self):

        nodes = [x for x in self.pomset.nodes() 
                 if x.name() == 'read tile index list']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]

        parameterId = 'temporal input'
        expectedParameterId = 'temporal input'

        # for an output parameter, itself
        nodeToEdit, parameterToEdit = node.getParameterToEdit(parameterId)

        self.assertTrue(nodeToEdit is node)
        self.assertEquals(expectedParameterId, parameterToEdit.id())

        parameterId = 'temporal output'
        expectedParameterId = 'temporal output'

        # for an output parameter, itself
        nodeToEdit, parameterToEdit = node.getParameterToEdit(parameterId)

        self.assertTrue(nodeToEdit is node)
        self.assertEquals(expectedParameterId, parameterToEdit.id())

        return


    def testParameterToEdit3(self):

        # for blackboard parameter, itself

        parameterId = 'blackboard for tile size'
        expectedParameterId = 'blackboard for tile size'

        parentDefinition = DefinitionModule.ReferenceDefinition()
        parentDefinition.definitionToReference(self.pomset)

        # for an output parameter, itself
        nodeToEdit, parameterToEdit = \
            parentDefinition.getParameterToEdit(parameterId)

        self.assertTrue(nodeToEdit is parentDefinition)
        self.assertEquals(expectedParameterId, parameterToEdit.id())


        parameterId = '%s.%s-%s.%s' % ('read tile index list',
                                       'eval result',
                                       'render tile',
                                       'tile index')
        expectedParameterId = parameterId

        nodeToEdit, parameterToEdit = \
            parentDefinition.getParameterToEdit(parameterId)

        self.assertTrue(nodeToEdit is parentDefinition)
        self.assertEquals(expectedParameterId, parameterToEdit.id())

        return


    # END class TestParameterToEdit1
    pass


class TestParameterToEdit2(unittest.TestCase):

    def testParameterToEdit1(self):

        # for an unconnected input parameter, itself
        
        raise NotImplementedError(
            'need to add a node that does not have a connected input parameter')


    # END class TestParameterToEdit2
    pass



class TestParameterToEdit3(unittest.TestCase):


    TEST_DATA_PATH = os.path.sep.join(['resources', 'testdata', 'TestDefinition', 'render.pomset'])

    def setUp(self):

        # load pomset from path
        pomsetContext = ContextModule.loadPomset(
            path=TestParameterToEdit3.TEST_DATA_PATH)

        self.pomset = pomsetContext.pomset()
        self.pomsetContext = pomsetContext
        return


    def testParameterToEdit1(self):
        """
        verify that exposed parameters return themselves
        """

        parameterId = 'tile size'
        expectedParameterId = 'tile size'

        parentDefinition = DefinitionModule.ReferenceDefinition()
        parentDefinition.definitionToReference(self.pomset)

        nodeToEdit, parameterToEdit = \
            parentDefinition.getParameterToEdit(parameterId)

        self.assertTrue(nodeToEdit is parentDefinition)
        self.assertEquals(expectedParameterId, parameterToEdit.id())
        

        return


    def testParameterToEdit2(self):
        """
        verify that a node's input parameter that's exposed
        returns the parent definition
        """

        nodes = [x for x in self.pomset.nodes()
                 if x.name() == 'render tile']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]

        parameterId = 'tile size'
        expectedParameterId = 'tile size'

        nodeToEdit, parameterToEdit = \
            self.pomsetContext.getParameterToEdit(node, parameterId)

        self.assertTrue(nodeToEdit is self.pomsetContext.reference())
        self.assertEquals(expectedParameterId, parameterToEdit.id())

        return


    def testParameterToEdit3(self):
        """
        verify that a node's output file parameter that's exposed
        returns the parent definition
        """
        nodes = [x for x in self.pomset.nodes()
                 if x.name() == 'comp tiles']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]

        parameterId = 'output file'
        expectedParameterId = 'output file'

        nodeToEdit, parameterToEdit = \
            self.pomsetContext.getParameterToEdit(node, parameterId)

        self.assertTrue(nodeToEdit is self.pomsetContext.reference())
        self.assertEquals(expectedParameterId, parameterToEdit.id())

        return


    def testParameterToEdit4(self):
        """
        verify that a node's output file parameter that's exposed
        returns the parent definition
        """
        nodes = [x for x in self.pomset.nodes()
                 if x.name() == 'comp tiles']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]

        parameterId = 'input dir'
        expectedParameterId = 'tmp dir'

        nodeToEdit, parameterToEdit = \
            self.pomsetContext.getParameterToEdit(node, parameterId)

        self.assertTrue(nodeToEdit is self.pomsetContext.reference())
        self.assertEquals(expectedParameterId, parameterToEdit.id())

        parameterId = 'output file'
        expectedParameterId = 'output file'

        nodeToEdit, parameterToEdit = \
            self.pomsetContext.getParameterToEdit(node, parameterId)

        self.assertTrue(nodeToEdit is self.pomsetContext.reference())
        self.assertEquals(expectedParameterId, parameterToEdit.id())

        return


    def testParameterToEdit5(self):
        """
        the output file connected to an input file
        is the parameter to edit
        """

        nodes = [x for x in self.pomset.nodes()
                 if x.name() == 'generate tile list']
        self.assertTrue(len(nodes) is 1)
        node = nodes[0]

        expectedNodes = [x for x in self.pomset.nodes()
                 if x.name() == 'generate tiling info']
        self.assertTrue(len(nodes) is 1)
        expectedNode = expectedNodes[0]

        parameterId = 'input file'
        expectedParameterId = 'output file'

        nodeToEdit, parameterToEdit = \
            node.getParameterToEdit(parameterId)

        self.assertTrue(nodeToEdit is expectedNode)
        self.assertEquals(expectedParameterId, parameterToEdit.id())


        return



    # END class TestParameterToEdit3
    pass




import os
import sys
import uuid

import optparse as OptparseModule

sys.path.insert(0, os.getenv('POMSETS_HOME'))

import pomsets.builder as BuilderModule
import pomsets.context as ContextModule
import pomsets.definition as DefinitionModule
import pomsets.library as LibraryModule
import pomsets.parameter as ParameterModule

import pomsets.test_utils as DefinitionTestModule




def generateBootstrapper():

    """
    defToLoadDef = DefinitionModule.AtomicDefinition()
    defToLoadDef.commandBuilderType('library bootstrap loader')
    defToLoadDef.executeEnvironmentType('library bootstrap loader')
    defToLoadDef.id(LibraryModule.ID_LOADLIBRARYDEFINITION)
    defToLoadDef.name('load library definition')
    """

    builder = BuilderModule.Builder()
    definitionContext = builder.createNewAtomicPomset(
        name='load library definition', 
        executableObject=None,
        commandBuilderType='library bootstrap loader',
        executeEnvironmentType='library bootstrap loader')
    definition = definitionContext.pomset()
    definition.id(LibraryModule.ID_LOADLIBRARYDEFINITION)

    """
    # need a command builder to call the loadPomset function
    # need a python eval environment to execute the output of commandbuilder
    parameter = ParameterModule.DataParameter(
        id='pomset url', optional=False, active=True,
        portDirection=ParameterModule.PORT_DIRECTION_INPUT)
    ParameterModule.setAttributes(parameter, {})
    defToLoadDef.addParameter(parameter)
    """
    builder.addPomsetParameter(
        definition, 'pomset url', 
        { 'direction':ParameterModule.PORT_DIRECTION_INPUT }
        )


    definition.isLibraryDefinition(True)
    definition.functionToExecute(DefinitionModule.executeTaskInEnvironment)
    return definition



def generateDefaultLoader(outputDir):

    definitionsToLoad = []

    defToLoadDef = generateBootstrapper()
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'loadLibraryDefinition.pomset'), defToLoadDef)
    context = ContextModule.wrapPomsetInContext(defToLoadDef)
    definitionsToLoad.append(context)
    
    wcDefinition = DefinitionTestModule.createWordCountDefinition()
    wcDefinitionPath = wcDefinition.id() + '.pomset'
    wcDefinition.url(wcDefinitionPath)
    ContextModule.pickleDefinition(
        os.path.join(outputDir, wcDefinitionPath), wcDefinition)
    context = ContextModule.wrapPomsetInContext(wcDefinition)
    definitionsToLoad.append(context)
    
    wcrDefinition = DefinitionTestModule.createWordCountReduceDefinition()
    wcrDefinitionPath = wcrDefinition.id() + '.pomset'
    wcrDefinition.url(wcrDefinitionPath)
    ContextModule.pickleDefinition(
        os.path.join(outputDir, wcrDefinitionPath), wcrDefinition)
    context = ContextModule.wrapPomsetInContext(wcrDefinition)
    definitionsToLoad.append(context)


    loadValuesDefinition = \
        DefinitionTestModule.createLoadListValuesFromFilesDefinition()
    loadValuesDefinitionPath = loadValuesDefinition.id() + '.pomset'
    loadValuesDefinition.url(loadValuesDefinitionPath)
    ContextModule.pickleDefinition(
        os.path.join(outputDir, loadValuesDefinitionPath), loadValuesDefinition)
    context = ContextModule.wrapPomsetInContext(loadValuesDefinition)
    definitionsToLoad.append(context)

    library = LibraryModule.Library()
    map(library.addPomsetContext, definitionsToLoad)

    defToLoadDefs = library.generateBootstrapLoaderPomset()
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'loadLibraryDefinitions.pomset'), defToLoadDefs)
    
    return


def generateLoaderWithFailure1(outputDir):
    
    definitionsToLoad = []

    defToLoadDef = generateBootstrapper()
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'loadLibraryDefinition.pomset'), defToLoadDef)
    context = ContextModule.wrapPomsetInContext(defToLoadDef)
    definitionsToLoad.append(context)
    
    wcDefinition = DefinitionTestModule.createWordCountDefinition()
    wcDefinitionPath = wcDefinition.id() + '.pomset'
    wcDefinition.url(wcDefinitionPath)
    # we purposely do not pickle it
    # to ensure that the loading fails
    context = ContextModule.wrapPomsetInContext(wcDefinition)
    definitionsToLoad.append(context)
    
    library = LibraryModule.Library()
    map(library.addPomsetContext, definitionsToLoad)

    defToLoadDefs = library.generateBootstrapLoaderPomset()
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'loadLibraryDefinitions.pomset'), defToLoadDefs)
    
    return



def generateLoaderWithFailure2(outputDir):

    definitionsToLoad = []

    defToLoadDef = generateBootstrapper()
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'loadLibraryDefinition.pomset'), defToLoadDef)
    context = ContextModule.wrapPomsetInContext(defToLoadDef)
    definitionsToLoad.append(context)
    
    wcDefinition = DefinitionTestModule.createWordCountDefinition()
    wcDefinitionPath = wcDefinition.id() + '.pomset'
    wcDefinition.url(wcDefinitionPath)
    # we purposely do not pickle it
    # to ensure that the loading fails
    context = ContextModule.wrapPomsetInContext(wcDefinition)
    definitionsToLoad.append(context)
    
    wcrDefinition = DefinitionTestModule.createWordCountReduceDefinition()
    wcrDefinitionPath = wcrDefinition.id() + '.pomset'
    wcrDefinition.url(wcrDefinitionPath)
    ContextModule.pickleDefinition(
        os.path.join(outputDir, wcrDefinitionPath), wcrDefinition)
    context = ContextModule.wrapPomsetInContext(wcrDefinition)
    definitionsToLoad.append(context)
    
    library = LibraryModule.Library()
    map(library.addPomsetContext, definitionsToLoad)

    defToLoadDefs = library.generateBootstrapLoaderPomset()
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'loadLibraryDefinitions.pomset'), defToLoadDefs)
    
    return



def main(args):

    parser = OptparseModule.OptionParser()
    parser.add_option("--default", dest="default",
                      help="if should generate default loader")
    parser.add_option("--failure1", dest="failure1",
                      help="if should generate loader with failure test case 1")
    parser.add_option("--failure2", dest="failure2",
                      help="if should generate loader with failure test case 2")
    parser.add_option("-o", "--output", dest="outputDir",
                      help="output dir")

    (options, args) = parser.parse_args(args)

    shouldGenerateDefaultLoader = options.default is not None
    shouldGenerateLoaderWithFailure1 = options.failure1 is not None
    shouldGenerateLoaderWithFailure2 = options.failure2 is not None

    outputDir = options.outputDir
    if outputDir is None:
        raise ValueError('need to specify directory to output the definitions')
        
    if shouldGenerateDefaultLoader:
        generateDefaultLoader(outputDir)
        
    if shouldGenerateLoaderWithFailure1:
        generateLoaderWithFailure1(outputDir)

    if shouldGenerateLoaderWithFailure2:
        generateLoaderWithFailure2(outputDir)
    return

if __name__=="__main__":
    main(sys.argv)




import unittest

import pypatterns.command as CommandPatternModule

import pomsets.automaton as AutomatonModule
import pomsets.builder as BuilderModule
import pomsets.task as TaskModule
import pomsets.test_utils as TestUtilsModule

class TestParameterSweepTaskGenerator(unittest.TestCase):

    def setUp(self):
        
        # create the automaton
        self.automaton = AutomatonModule.Automaton()
        self.automaton.commandManager(CommandPatternModule.CommandManager())


        # create the generator
        self.generator = TaskModule.ParameterSweepTaskGenerator()

        # create the pomset
        self.pomset = TestUtilsModule.createPomsetContainingParameterSweep()
        TestUtilsModule.bindParameterSweepDefinitionParameters(self.pomset)

        return

    def tearDown(self):
        return

    def testGenerateTasks1(self):

        parentDefinition = [x for x in self.pomset.nodes()
                            if x.id() == 'mapper'][0]
        

        # create the parent task
        parentTask = TaskModule.CompositeTask()
        parentTask.definition(parentDefinition)
        parentTask.automaton(self.automaton)

        parentTask.pullParameterBindingsFromDefinition()

        self.generator.generateReadyTasks(parentTask)

        self.assertEquals(2,
                          self.generator.taskTable().rowCount())

        expectedValueMap = {
            0:{
                'input file':['/tmp/pomsets/text1'],
                'output file':['/tmp/pomsets/count1']
                },
            1:{
                'input file':['/tmp/pomsets/text2'],
                'output file':['/tmp/pomsets/count2']
                },
            }


        # now assert the generated tasks
        for row in self.generator.taskTable().rows():
            index = row.getColumn('index')
            expectedValues = expectedValueMap[index]
            parameterBindings = row.getColumn('parameter bindings')
            for key, expectedBinding in parameterBindings.iteritems():
                self.assertEquals(expectedBinding,
                                  parameterBindings[key])

        

        return


    def testGenerateTasks2(self):

        parentDefinition = [x for x in self.pomset.nodes()
                            if x.id() == 'mapper'][0]
        
        parentDefinition.removeParameterSweepGroup(
            ['input file', 'output file'])

        # create the parent task
        parentTask = TaskModule.CompositeTask()
        parentTask.definition(parentDefinition)
        parentTask.automaton(self.automaton)

        parentTask.pullParameterBindingsFromDefinition()

        self.generator.generateReadyTasks(parentTask)

        self.assertEquals(4,
                          self.generator.taskTable().rowCount())

        expectedValueMap = {
            0:{
                'input file':['/tmp/pomsets/text1'],
                'output file':['/tmp/pomsets/count1']
                },
            1:{
                'input file':['/tmp/pomsets/text2'],
                'output file':['/tmp/pomsets/count2']
                },
            2:{
                'input file':['/tmp/pomsets/text1'],
                'output file':['/tmp/pomsets/count2']
                },
            3:{
                'input file':['/tmp/pomsets/text2'],
                'output file':['/tmp/pomsets/count1']
                },
            }


        # now assert the generated tasks
        for row in self.generator.taskTable().rows():
            index = row.getColumn('index')
            expectedValues = expectedValueMap[index]
            parameterBindings = row.getColumn('parameter bindings')
            for key, expectedBinding in parameterBindings.iteritems():
                self.assertEquals(expectedBinding,
                                  parameterBindings[key])

        

        return





    # END class TestTaskGenerator
    pass



import imp
import logging
import os

import cloudpool.environment as EnvironmentModule
import cloudpool.task as TaskModule

import pypatterns.filter as FilterModule

import pomsets.command as CommandModule
import pomsets.parameter as ParameterModule

class Function(CommandModule.Executable):

    ATTRIBUTES = CommandModule.Executable.ATTRIBUTES + []

    def __init__(self):
        CommandModule.Executable.__init__(self)
        self.path(['',''])
        return

    def name(self):
        return self.path()[1]

    def module(self):
        return self.path()[0]

    # END class Function
    pass


class CommandBuilder(TaskModule.CommandBuilder):

    def buildCommand(self, task):
        workRequest = task.workRequest()

        
        # get the function name
        functionName = task.definition().executable().name()
        

        parameterBindings = task.parameterBindings()
    

        parameterFilter = task.definition().getFilterForCommandlineArguments()
    
        parameters = [
            x for x in
            task.definition().getParametersByFilter(parameterFilter)]

        parameters = ParameterModule.sortParameters(
            parameters, 
            task.definition().parameterOrderingTable())

        commandArgList = []
        for parameter in parameters:
            key = parameter.id()

            value = task.getParameterBinding(key)
            if isinstance(value, str):
                # TODO
                # this should attempt to escape certain chars?
                value = '"%s"' % value
            else:
                value = str(value)

            # TODO:
            # need to determine if should pass as keyword
            # and if so, need to retrieve the keyword
            if parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_KEYWORD):
                keyword = parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_KEYWORDTOPASS)
                value = '='.join([keyword, value])

            commandArgList.append(value)
            pass

        commandList = []
        commandList.append(functionName)
        commandList.append('(')
        commandList.append(', '.join(commandArgList))
        commandList.append(')')

        command = ''.join(commandList)
        return command
    
    # END class CommandBuilder
    pass


class PythonEval(EnvironmentModule.Environment):

    DEFAULT_COMMANDBUILDER_TYPE = 'python eval'


    @staticmethod
    def importModule(name):

        modulePath = name.split('.')
        module = __import__(modulePath[0])

        if len(modulePath) > 1:
            module = __import__(name, fromlist=modulePath[:1])

        return module

    
    def execute(self, task, *args, **kargs):

        request = task.workRequest()
        
        commandBuilder = self.getCommandBuilder(task)

        command = commandBuilder.buildCommand(task)

        request.kwds['executed command'] = command
        
        logging.debug('%s executing command "%s"' % (self.__class__, command))

        # evalResult = eval(command)
        moduleName = task.definition().executable().module()
        if moduleName is not None:
            module = PythonEval.importModule(moduleName)
            command = '.'.join(['module', command])

        evalResult = eval(command)

        request.kwds['eval result'] = evalResult
        task.setParameterBinding('eval result', evalResult)

        return 0
    
    # END class PythonEval
    pass



import os

import currypy

import cloudpool.task as TaskModule
import cloudpool.environment as ExecuteEnvironmentModule

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule
import pomsets.resource as ResourceModule

import pomsets.parameter as ParameterModule




def buildCommandFunction_default(task):
    """
    This just looks for the key "command" in kwds
    and sets that as the command
    """
    
    kwds = task.parameterBindings()
    command = kwds['command']
    return command


def buildCommandFunction_commandlineArgs(task):

    command = [os.path.sep.join(task.definition().executable().path())] + \
            task.definition().executable().staticArgs() + \
            buildCommandFunction_commandlineArgsOnly(task)
    
    execPath = task.definition().executable().path()

    return command

    
def buildCommandFunction_commandlineArgsOnly(task):
    
    parameterBindings = task.parameterBindings()
    
    commandlineParameterFilter = \
        task.definition().getFilterForCommandlineArguments()

    commandLineParameters = [
        x for x in
        task.definition().getParametersByFilter(commandlineParameterFilter)]

    commandLineParameters = ParameterModule.sortParameters(
        commandLineParameters, 
        task.definition().parameterOrderingTable())
    
    command = []
    for parameter in commandLineParameters:

        command.extend(buildParameterArgument(parameter, parameterBindings))
        
        pass
            
    return command


def buildParameterArgument(parameter, parameterBindings):
    argument = []
    
    # TODO:
    # need to be able to parameterize how the parameter is output
    # currently just outputs the prefix flag
    # and the value placed in parameterBindings for parameter
    
    commandlineOptions = parameter.getAttribute(
        ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS,
        defaultValue = {}
    )
    
    prefixFlag = commandlineOptions.get(
        ParameterModule.COMMANDLINE_PREFIX_FLAG, []
    )
    argumentValue = parameterBindings[parameter.id()]
    
    argumentValueIsList = parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISLIST)
    shouldDistributePrefixFlag = commandlineOptions.get(
        ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE, False)
    
    if argumentValueIsList and shouldDistributePrefixFlag:
        # TODO:
        # should call some function recursively
        # so that it can handle the value appropriately
        
        map(argument.extend, [prefixFlag+[x] for x in argumentValue])
    else:
        argumentValueIsEnum = parameter.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISENUM)
        if argumentValueIsEnum:
            enumMap = commandlineOptions.get(
                ParameterModule.COMMANDLINE_ENUM_MAP, {}
            )
            # TODO:
            # verify that there's only one item in argumentValue?
            argumentValue = enumMap.get(argumentValue[0], [])
            pass
        
        argument.extend(prefixFlag)
        argument.extend(argumentValue)
        pass
    
    return argument


class CommandBuilder(TaskModule.CommandBuilder):


    def __init__(self, 
                 buildCommandFunction=None):
        
        TaskModule.CommandBuilder.__init__(self)
        
        if buildCommandFunction is None:
            buildCommandFunction = buildCommandFunction_default

        self.buildCommand = buildCommandFunction

        return

    # END class CommandBuilder
    pass


class DelegatingCommandBuilder(CommandBuilder):
    
    def __init__(self, buildCommandFunction=None):

        self._delegate = \
            CommandBuilder(buildCommandFunction=buildCommandFunction)
        
        return
    
    def buildCommand(self, task):
        command = self._delegate.buildCommand(task)
        
        command = self.wrapDelegateCommand(task, command)
        return command
    
    # END class DelegatingCommandBuilder
    pass



class Executable(ResourceModule.Struct):

    # TODO:
    # in the future, whether something is stageable
    # will be determined by whether its dependencies
    # are all stageable
    ATTRIBUTES = [
        'stageable', 'path', 'staticArgs'
    ]
    
    def __init__(self):
        ResourceModule.Struct.__init__(self)
        return
    
    # END class Executable
    pass



class PrintTaskCommand(ExecuteEnvironmentModule.Environment, ResourceModule.Struct):
    
    DEFAULT_COMMANDBUILDER_TYPE = 'print task'
    
    ATTRIBUTES = [
        'outputStream',
        'prefix',
        'postfix'
        ]

    def __init__(self):
        ResourceModule.Struct.__init__(self)
        return
    
    def execute(self, task, *args, **kargs):
        
        commandBuilder = self.getCommandBuilder(task)
        
        command = commandBuilder.buildCommand(task)

        if self.outputStream():
            if self.prefix():
                self.outputStream().write(self.prefix())

            self.outputStream().write(task.definition().name() + '\n')
            self.outputStream().write(' '.join(command))

            if self.postfix():
                self.outputStream().write(self.postfix())
        
        return 0

    
    # END class PrintTaskCommand
    pass



import os
import unittest

import pypatterns.command as CommandPatternModule
import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

import cloudpool as CloudModule

import pomsets.automaton as AutomatonModule
import pomsets.builder as BuilderModule
import pomsets.library as LibraryModule
import pomsets.parameter as ParameterModule
import pomsets.python as PythonModule
import pomsets.task as TaskModule
import pomsets.test_utils as TestUtilsModule


import TestExecute as BaseModule
import TestLoadLibraryReference as TestLibraryModule

class TestPythonFunction(unittest.TestCase):

    def setUp(self):
        self.builder = BuilderModule.Builder()
        return

    def tearDown(self):
        return
    
    def testGetFunctionName(self):

        functionName = 'testFunction'


        executable = self.builder.createExecutableObject(
            ['', functionName],
            executableClass=PythonModule.Function)

        self.assertEquals(functionName, executable.name())

        return

    # END class TestPythonFunction
    pass


class TestPythonCommandBuilder(unittest.TestCase):


    def setUp(self):
        self.builder = BuilderModule.Builder()

        self.functionName = 'testFunction'
        self.executableObject = self.builder.createExecutableObject(
            ['', self.functionName],
            executableClass=PythonModule.Function)

        parentContext = self.builder.createNewNestPomset(name='root')
        parentDefinition = parentContext.pomset()
        
        pythonEvalContext = self.builder.createNewAtomicPomset(
            name='testPython',
            executableObject = self.executableObject,
            commandBuilderType = 'python eval')
        pythonEvalDefinition = pythonEvalContext.pomset()
        self.definitionToReference = pythonEvalDefinition

        self.definition = self.builder.createNewNode(
            parentDefinition, name='node', 
            definitionToReference=pythonEvalDefinition)

        self.commandBuilder = PythonModule.CommandBuilder()

        parentTask = TaskModule.CompositeTask()
        parentTask.definition(parentDefinition)
        taskGenerator = TaskModule.NestTaskGenerator()
        parentTask.taskGenerator(taskGenerator)
        self.parentTask = parentTask

        return


    def tearDown(self):
        return


    def assertBuiltCommand(self, expected):
        task = TaskModule.createTaskForDefinition(
            self.parentTask, self.definition)

        command = self.commandBuilder.buildCommand(task)

        self.assertEquals(expected,
                          command)

        return


    def testBuildCommand1(self):
        """
        this ensures that building succeeds
        when there are no arguments
        """

        self.assertBuiltCommand('%s()' % self.functionName)
        return


    def testBuildCommandArgOnly1(self):
        # this ensures that building succeeds
        # when there is only a single non-keyword arg

        parameterId = 'arg1'

        # add parameters to definition
        self.builder.addPomsetParameter(
            self.definitionToReference, parameterId,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT})

        arg1 = 'arg1'
        self.builder.bindParameterValue(self.definition,
                                        parameterId,
                                        arg1)


        self.assertBuiltCommand('%s("%s")' % (self.functionName, arg1))
        return


    def testBuildCommandArgOnly2(self):
        # this ensures that building succeeds
        # when there are only non-keyword args

        parameter1Id = 'arg1'
        parameter2Id = 'arg2'

        # add parameters to definition
        self.builder.addPomsetParameter(
            self.definitionToReference, parameter1Id,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT})
        self.builder.addPomsetParameter(
            self.definitionToReference, parameter2Id,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT})

        # order parameters
        self.builder.addParameterOrdering(
            self.definitionToReference, 
            parameter1Id, parameter2Id)


        # bind values on task
        arg1 = 'arg1'
        arg2 = 5

        self.builder.bindParameterValue(self.definition,
                                        parameter1Id,
                                        arg1)
        self.builder.bindParameterValue(self.definition,
                                        parameter2Id,
                                        arg2)


        self.assertBuiltCommand('%s("%s", %s)' % 
                                (self.functionName, arg1, arg2))
        return


    def testBuildCommandKeywordOnly1(self):
        # this ensures that building succeeds
        # when there are only a single keyword arg

        parameterId = 'arg1'
        arg1Key = 'arg1key'
 
        # add parameters to definition
        self.builder.addPomsetParameter(
            self.definitionToReference, parameterId,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT,
             'is keyword':True,
             'keyword to pass':arg1Key})

        arg1 = 'arg1value'
        self.builder.bindParameterValue(self.definition,
                                        parameterId,
                                        arg1)


        self.assertBuiltCommand('%s(%s="%s")' % 
                                (self.functionName, arg1Key, arg1))
        return


    def testBuildCommandKeywordOnly2(self):
        # this ensures that building succeeds
        # when there are only keyword args

        parameter1Id = 'arg1'
        parameter2Id = 'arg2'
        parameter3Id = 'arg3'

        arg1Key = 'arg1key'
        arg2Key = 'arg2key'
        arg3Key = 'arg3key'
 
        # add parameters to definition
        self.builder.addPomsetParameter(
            self.definitionToReference, parameter1Id,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT,
             'is keyword':True,
             'keyword to pass':arg1Key})
        self.builder.addPomsetParameter(
            self.definitionToReference, parameter2Id,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT,
             'is keyword':True,
             'keyword to pass':arg2Key})
        self.builder.addPomsetParameter(
            self.definitionToReference, parameter3Id,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT,
             'is keyword':True,
             'keyword to pass':arg3Key})

        # order parameters
        self.builder.addParameterOrdering(
            self.definitionToReference, 
            parameter2Id, parameter1Id)
        self.builder.addParameterOrdering(
            self.definitionToReference, 
            parameter1Id, parameter3Id)

        arg1 = 'arg1value'
        arg2 = 8
        arg3 = None

        self.builder.bindParameterValue(self.definition,
                                        parameter1Id,
                                        arg1)
        self.builder.bindParameterValue(self.definition,
                                        parameter2Id,
                                        arg2)
        self.builder.bindParameterValue(self.definition,
                                        parameter3Id,
                                        arg3)


        self.assertBuiltCommand('%s(%s=%s, %s="%s", %s=%s)' % 
                                (self.functionName, 
                                 arg2Key, arg2,
                                 arg1Key, arg1,
                                 arg3Key, arg3))
        return


    def testBuildCommand6(self):
        # this ensures that building succeeds
        # when there are both keyword and non-keyword args

        parameter1Id = 'arg1'
        parameter2Id = 'arg2'
        parameter3Id = 'arg3'

        arg1Key = 'arg1key'
        arg3Key = 'arg3key'
 
        # add parameters to definition
        self.builder.addPomsetParameter(
            self.definitionToReference, parameter1Id,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT,
             'is keyword':True,
             'keyword to pass':arg1Key})
        self.builder.addPomsetParameter(
            self.definitionToReference, parameter2Id,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT})
        self.builder.addPomsetParameter(
            self.definitionToReference, parameter3Id,
            {'direction':ParameterModule.PORT_DIRECTION_INPUT,
             'is keyword':True,
             'keyword to pass':arg3Key})

        # order parameters
        self.builder.addParameterOrdering(
            self.definitionToReference, 
            parameter2Id, parameter1Id)
        self.builder.addParameterOrdering(
            self.definitionToReference, 
            parameter1Id, parameter3Id)

        arg1 = 'arg1value'
        arg2 = 8
        arg3 = None

        self.builder.bindParameterValue(self.definition,
                                        parameter1Id,
                                        arg1)
        self.builder.bindParameterValue(self.definition,
                                        parameter2Id,
                                        arg2)
        self.builder.bindParameterValue(self.definition,
                                        parameter3Id,
                                        arg3)


        self.assertBuiltCommand('%s(%s, %s="%s", %s=%s)' % 
                                (self.functionName, 
                                 arg2,
                                 arg1Key, arg1,
                                 arg3Key, arg3))

        return


    # END class 
    pass





class TestLoadListValues1(BaseModule.BaseTestClass, unittest.TestCase):


    BASE_DIR = os.path.join(os.getcwd(), 'resources', 'testdata', 'TestOperator')
    INPUT_FILE = 'listValues1'


    def setUp(self):
        BaseModule.BaseTestClass.setUp(self)
        self.builder = BuilderModule.Builder()
        return


    def getPythonEvalDefinition(self):
        pythonEvalDefinition = \
            TestUtilsModule.createLoadListValuesFromFilesDefinition()
        return pythonEvalDefinition


    def createDefinition(self):
        pythonEvalDefinition = self.getPythonEvalDefinition()

        parentContext = self.builder.createNewNestPomset(name='root')
        parentDefinition = parentContext.pomset()
        

        self.definition = self.builder.createNewNode(
            parentDefinition, name='node', 
            definitionToReference=pythonEvalDefinition)


        self.builder.bindParameterValue(
            self.definition,
            'file to read',
            [os.path.sep.join([TestLoadListValues1.BASE_DIR, 
                               TestLoadListValues1.INPUT_FILE])]
            )

        return parentDefinition


    def createCommandBuilderMap(self):
        commandBuilderMap = BaseModule.BaseTestClass.createCommandBuilderMap(self)
        commandBuilderMap['python eval'] = PythonModule.CommandBuilder()
        return commandBuilderMap


    def createExecuteEnvironmentMap(self):
        return {
            'python eval':PythonModule.PythonEval()
            }


    def createTask(self, definition):

        parentTask = TaskModule.CompositeTask()
        parentTask.definition(definition)
        taskGenerator = TaskModule.NestTaskGenerator()
        parentTask.taskGenerator(taskGenerator)

        # TODO:
        # bind the parameters
        #task = TaskModule.createTaskForDefinition(
        #    parentTask, self.definition)
        # return task

        self.parentTask = parentTask

        return parentTask


    def getPicklePath(self):
        return os.path.sep + \
            os.path.join('tmp', 'TestOperator.%s.testExecute2' % self.__class__.__name__)


    def assertPostExecute(self):
        BaseModule.BaseTestClass.assertPostExecute(self)

        childTasks = self.parentTask.getChildTasks()
        childTask = childTasks[0]

        values = childTask.getParameterBinding('eval result')

        self.assertEquals(['/tmp/foo', '/user/home/me/bar.txt', './baz.ext'],
                          values)
        
        return 


    # END class TestLoadListValues1
    pass


class TestLoadListValues2(TestLoadListValues1):


    def getLibraryDir(self):
        return os.path.join(os.getcwd(), 'resources', 'testdata', 'TestLibrary', 'library')

    def initializeLibrary(self):
        library = LibraryModule.Library()
        library.shouldMarkAsLibraryDefinition(True)
        libraryDir = self.getLibraryDir()
        library.bootstrapLoaderDefinitionsDir(libraryDir)

        library.loadBootstrapLoaderDefinitions()
        return library

    def setUp(self):
        TestLoadListValues1.setUp(self)

        automaton = AutomatonModule.Automaton()
        automaton.setThreadPool(None, CloudModule.Pool(1))
        automaton.commandManager(CommandPatternModule.CommandManager())
        self.automaton = automaton

        self.library = self.initializeLibrary()
        automaton.runBootstrapLoader(self.library)
        return


    def getPythonEvalDefinition(self):
            
        filter = RelationalModule.ColumnValueFilter(
            'definition',
            FilterModule.IdFilter(LibraryModule.ID_LOADLISTVALUESFROMFILES))
        
        return self.library.getDefinition(filter)

        
    # END class TestLoadListValues2
    pass


class TestRange1(BaseModule.BaseTestClass, unittest.TestCase):


    def setUp(self):
        BaseModule.BaseTestClass.setUp(self)
        self.builder = BuilderModule.Builder()
        return


    def getPythonEvalDefinition(self):
        pythonEvalDefinition = \
            TestUtilsModule.createRangeDefinition()
        return pythonEvalDefinition


    def createDefinition(self):
        pythonEvalDefinition = self.getPythonEvalDefinition()

        parentContext = self.builder.createNewNestPomset(name='root')
        parentDefinition = parentContext.pomset()
        

        self.definition = self.builder.createNewNode(
            parentDefinition, name='node', 
            definitionToReference=pythonEvalDefinition)


        self.bindParameterValues()

        return parentDefinition


    def bindParameterValues(self):
        self.builder.bindParameterValue(
            self.definition,
            'end', 3)

        return


    def createCommandBuilderMap(self):
        commandBuilderMap = BaseModule.BaseTestClass.createCommandBuilderMap(self)
        commandBuilderMap['python eval'] = PythonModule.CommandBuilder()
        return commandBuilderMap


    def createExecuteEnvironmentMap(self):
        return {
            'python eval':PythonModule.PythonEval()
            }


    def createTask(self, definition):

        parentTask = TaskModule.CompositeTask()
        parentTask.definition(definition)
        taskGenerator = TaskModule.NestTaskGenerator()
        parentTask.taskGenerator(taskGenerator)

        # TODO:
        # bind the parameters
        #task = TaskModule.createTaskForDefinition(
        #    parentTask, self.definition)
        # return task

        self.parentTask = parentTask

        return parentTask


    def getPicklePath(self):
        return os.path.sep + \
            os.path.join('tmp', 'TestOperator.%s.testExecute2' % self.__class__.__name__)


    def assertPostExecute(self):
        BaseModule.BaseTestClass.assertPostExecute(self)

        childTasks = self.parentTask.getChildTasks()
        childTask = childTasks[0]

        values = childTask.getParameterBinding('eval result')

        self.assertEquals(range(3),
                          values)
        
        return 


    # END class TestRange1
    pass


class TestRange2(TestRange1):

    def bindParameterValues(self):
        self.builder.bindParameterValue(
            self.definition,
            'start', 2)

        self.builder.bindParameterValue(
            self.definition,
            'end', 10)

        self.definition.parameterIsActive('start', True)

        return

    def assertPostExecute(self):
        BaseModule.BaseTestClass.assertPostExecute(self)

        childTasks = self.parentTask.getChildTasks()
        childTask = childTasks[0]

        values = childTask.getParameterBinding('eval result')

        self.assertEquals(range(2,10),
                          values)
        
        return 


    # END class TestRange2
    pass


class TestRange3(TestRange1):
    """
    This tests the default value
    """

    def getPythonEvalDefinition(self):
        pythonEvalDefinition = TestRange1.getPythonEvalDefinition(self)

        parameter = pythonEvalDefinition.getParameter('start')
        parameter.defaultValue(5)

        return pythonEvalDefinition


    def bindParameterValues(self):

        self.builder.bindParameterValue(
            self.definition,
            'end', 10)

        self.definition.parameterIsActive('start', True)

        return


    def assertPostExecute(self):
        BaseModule.BaseTestClass.assertPostExecute(self)

        childTasks = self.parentTask.getChildTasks()
        childTask = childTasks[0]

        values = childTask.getParameterBinding('eval result')

        self.assertEquals(range(5,10),
                          values)
        
        return 


    # END class TestRange3
    pass


class TestRange4(TestRange1):

    def bindParameterValues(self):
        self.builder.bindParameterValue(
            self.definition,
            'start', 2)

        self.builder.bindParameterValue(
            self.definition,
            'end', 10)

        self.builder.bindParameterValue(
            self.definition,
            'step', 3)

        self.definition.parameterIsActive('start', True)
        self.definition.parameterIsActive('step', True)

        return

    def assertPostExecute(self):
        BaseModule.BaseTestClass.assertPostExecute(self)

        childTasks = self.parentTask.getChildTasks()
        childTask = childTasks[0]

        values = childTask.getParameterBinding('eval result')

        self.assertEquals(range(2,10,3),
                          values)
        
        return 


    # END class TestRange4
    pass




import copy
import logging
import uuid

import pypatterns.relational as RelationalModule
import pypatterns.filter as FilterModule

import pomsets.command as TaskCommandModule
import pomsets.context as ContextModule
import pomsets.definition as DefinitionModule
import pomsets.parameter as ParameterModule

class Builder(object):


    def bindParameterValue(self, node, parameterId, value):
        (nodeToEdit, parameterToEdit) = \
            node.getParameterToEdit(parameterId)
        nodeToEdit.setParameterBinding(parameterToEdit.id(), value)
        return

    def removePomsetParameter(self, pomset, parameterName):

        if parameterName in [DefinitionModule.Definition.SYMBOL_INPUT_TEMPORAL,
                             DefinitionModule.Definition.SYMBOL_OUTPUT_TEMPORAL]:
            raise ValueError('cannot remove parameters defined by the system')

        parameter = pomset.getParameter(parameterName)
        pomset.removeParameter(parameter)
        return
        
    def addPomsetParameter(self, pomset, parameterName, attributes):

        # TODO:
        # should verify that the pomset
        # does not already have a parameter with the given name


        # the direction has to be specified
        # almost everything else has a default value
        direction = attributes['direction']

        isOptional = attributes.get('optional', False)
        isActive = attributes.get('active', not isOptional)
        isCommandline = attributes.get('commandline', True)
        isFile = attributes.get('file', False)
        isList = attributes.get('list', False)
        isEnum = attributes.get('enum', False)

        commandlineOptions = {}
        if isCommandline:
            prefixFlag = attributes.get('prefix flag', [])
            distributePrefixFlag = attributes.get(
                'distribute prefix flag', False)
            
            enumMap = attributes.get('enum map', {})

            commandlineOptions = {
                ParameterModule.COMMANDLINE_PREFIX_FLAG:prefixFlag,
                ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE:distributePrefixFlag,
                ParameterModule.COMMANDLINE_ENUM_MAP:enumMap
                }
            pass

        isInputFile = (direction == ParameterModule.PORT_DIRECTION_INPUT and 
                       isFile)

        isSideEffect = (direction == ParameterModule.PORT_DIRECTION_OUTPUT and
                        isFile)
        if isSideEffect:
            # we have to do this because the output is the file
            # but the name of the file is actually the input
            direction = ParameterModule.PORT_DIRECTION_INPUT

        isKeyword = attributes.get('is keyword', False)
        keywordToPass = ''
        if isKeyword:
            keywordToPass = attributes.get('keyword to pass')

        parameterAttributes = {
            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:isCommandline,
            ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE:isInputFile,
            ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT:isSideEffect,
            ParameterModule.PORT_ATTRIBUTE_ISLIST:isList,
            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS:commandlineOptions,
            ParameterModule.PORT_ATTRIBUTE_KEYWORD:isKeyword,
            ParameterModule.PORT_ATTRIBUTE_KEYWORDTOPASS:keywordToPass,
            }

        parameter = ParameterModule.DataParameter(
            id=parameterName, 
            optional=isOptional, active=isActive,
            portDirection=direction)
        parameter.name(parameterName)

        defaultValue = attributes.get('default value', None)
        if defaultValue is not None:
            parameter.defaultValue(defaultValue)

        ParameterModule.setAttributes(parameter, parameterAttributes)

        pomset.addParameter(parameter)
        pomset.parameterIsActive(parameterName, isActive)

        return parameter


    def addParameterOrdering(self, pomset, sourceParameterName, targetParameterName):
        parameterOrderings = pomset.parameterOrderingTable()
        row = parameterOrderings.addRow()
        row.setColumn('source', sourceParameterName)
        row.setColumn('target', targetParameterName)
        return


    def createExecutableObject(self, path, 
                               executableClass=None, staticArgs=None):

        if executableClass is None:
            executableClass = TaskCommandModule.Executable
        executableObject = executableClass()

        executableObject.stageable(False)
        executableObject.path(path)
        if staticArgs is None:
            staticArgs = []
        executableObject.staticArgs(staticArgs)
        return executableObject


    def createNewAtomicPomset(self, name=None, 
                              executableObject=None,
                              commandBuilderType=None,
                              executeEnvironmentType=None,
                              *args, **kwds):

        newAtomicPomset = DefinitionModule.AtomicDefinition(*args, **kwds)
        if name is None:
            name = 'pomset %s' % uuid.uuid4().hex[:3]
        newAtomicPomset.name(name)

        newAtomicPomset.functionToExecute(
            DefinitionModule.executeTaskInEnvironment)

        newAtomicPomset.executable(executableObject)

        # create the parameter orderings
        parameterOrderings = DefinitionModule.createParameterOrderingTable()
        newAtomicPomset.parameterOrderingTable(parameterOrderings)

        if commandBuilderType is None:
            commandBuilderType = 'shell process'
        newAtomicPomset.commandBuilderType(commandBuilderType)

        if executeEnvironmentType is None:
            executeEnvironmentType = 'shell process'
        newAtomicPomset.executeEnvironmentType(executeEnvironmentType)

        newPomsetContext = ContextModule.wrapPomsetInContext(newAtomicPomset)
        
        return newPomsetContext




    def createNewNestPomset(self, name=None):
        """
        """
        #TODO: this should construct a command to create the new pomset
        #      and execute the command (or send an event to execute the command)
        #      within the command framework, the command should also
        #      create an event to update the GUI

        newPomset = DefinitionModule.getNewNestDefinition()

        if name is None:
            name = 'pomset %s' % uuid.uuid4().hex[:3]
        newPomset.name(name)
        
        newPomsetContext = ContextModule.wrapPomsetInContext(newPomset)
        
        return newPomsetContext


    def copyNode(self, pomset, node, name=None, 
                 shouldCopyEdges=False,
                 shouldCopyParameterBindings=False):

        # create a node that references the same definition
        # copy edges if necessary
        # copy parameter bindings if necessary
        nodeCopy = copy.copy(node)
        pomset.addNode(nodeCopy)

        if shouldCopyEdges:
            raise NotImplementedError('need to implement shouldCopyEdges')

        if shouldCopyParameterBindings:
            raise NotImplementedError('need to implement shouldCopyParameterBindings')

        return nodeCopy


    def createNewNode(self, pomset, name=None,
                      definitionToReference=None):

        #TODO: this should construct a command to create the new node
        #      and execute the command (or send an event to execute the command)
        #      within the command framework, the command should also
        #      create an event to update the GUI

        if name is None:
            name = 'job %s' % len(pomset.nodes())

        id = uuid.uuid4().hex
        node = pomset.createNode(id=id)
        node.name(name)

        if definitionToReference is None:
            raise ValueError("need to specify a definition to reference")

        node.definitionToReference(definitionToReference)

        # TODO:
        # see if it's possible to not have to run this line
        node.executable = definitionToReference.executable

        return node


    def removeNode(self, pomset, node, maintainTransitivity=False):
        if maintainTransitivity:
            raise NotImplementedError

        # first remove all the incoming and outgoing connections
        filter = FilterModule.constructOrFilter()
        filter.addFilter(
            RelationalModule.ColumnValueFilter(
                'source node',
                FilterModule.IdentityFilter(node)
                )
            )
        filter.addFilter(
            RelationalModule.ColumnValueFilter(
                'target node',
                FilterModule.IdentityFilter(node)
                )
            )
        
        columns = ['source node', 'source parameter',
                   'target node', 'target parameter']
        rowValueList = [x for x in 
                pomset.parameterConnectionPathTable().retrieve(
                filter=filter, columns=columns)]

        for rowValues in rowValueList:
            self.disconnect(pomset, *rowValues)
            pass

        pomset.removeNode(node)

        return


    def canConnect(self, 
                   pomset,
                   sourceNode, sourceParameterId,
                   targetNode, targetParameterId):
        """
        This is a validation function to determine whether the
        ports provided can be connected to each other
        """
        return pomset.canConnect(sourceNode, sourceParameterId,
                                 targetNode, targetParameterId)


    def connect(self, 
                pomset,
                sourceNode, sourceParameterId,
                targetNode, targetParameterId):
        """
        This assumes that the caller has already
        verified that canConnect() returns True
        """
        return pomset.connectNodes(sourceNode, sourceParameterId,
                                   targetNode, targetParameterId)



    def disconnect(self, pomset,
                   sourceNode, sourceParameterId,
                   targetNode, targetParameterId):
        """
        looks for the connection path
        then removes the individual atomic connections
        """
        return pomset.disconnect(sourceNode, sourceParameterId,
                                 targetNode, targetParameterId)


    def exposeNodeParameter(self, pomset, parameterId,
                            node, nodeParameterId,
                            shouldCreate=False):
        return pomset.exposeNodeParameter(
            parameterId, node, nodeParameterId, 
            shouldCreate=shouldCreate)


    # END class Builder
    pass




from __future__ import with_statement

def loadListValuesFromFiles(filePaths):

    if type(filePaths) is str:
        filePaths = [filePaths]

    values = []

    for filePath in filePaths:

        with open(filePath) as f:

            for line in f.readlines():
                line = line.strip()
                if len(line):
                    values.append(line)
                pass
            pass
        pass

    return values




import cloudpool.environment as EnvironmentModule

import pomsets.resource as ResourceModule


from celery.decorators import task


# TODO:
# implement this decorated function execute the task
@task
def execute_task(env, task, *args, **kwds):
    env.execute(task, *args, **kwds)
    return


class Pool(ResourceModule.Struct):

    ATTRIBUTES = [
        'workRequests'
        ]

    def __init__(self):
        ResourceModule.Struct.__init__(self)
        self.workRequests({})
        return

    def isEmpty(self):
        # TODO:
        # need a way to determine if there are nodes
        # in the celery cluster
        return False

    def putRequest(self, request, block=True, timeout=None):
        """
        Put work request into work queue and save its id for later.

        NOTE: most of this function duplicates threadpool.ThreadPool.putRequest
        except for the actually putting into the request queue
        """

        assert isinstance(request, WorkRequest)
        # don't reuse old work requests
        assert not getattr(request, 'exception', None)

        # self._requests_queue.put(request, block, timeout)
        # TODO:
        if True:
            raise NotImplementedError('putting the request in the queue')

        self.workRequests()[request.requestID] = request


    # END class Pool
    pass


class Environment(EnvironmentModule.Environment, ResourceModule.Struct):
    """
    This celery.Environment class executes 
    locally on the pomsets app host,
    while celery.Environment.environment() 
    will execute on the celery worker host
    """
    
    ATTRIBUTES = [
        'environment'
        ]

    def __init__(self):
        EnvironmentModule.Environment.__init__(self)
        ResourceModule.Struct.__init__(self)
        return

    def execute(self, task, *args, **kargs):
        result = execute_task.delay(self.environment(), task, *args, **kwds)

        # this waits for the execution
        returnValue = result.get()
        if not result.successful():
            raise result.result
        return returnValue

    # END class Environment
    pass
